from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        uprn = kwargs.get("uprn")
        check_uprn(uprn)

        # Figure bin data URL from UPRN
        url = "https://www.northyorks.gov.uk/bin-calendar/lookup"

        payload = {
            "selected_address": uprn,
            "submit": "Continue",
            "form_id": "bin_calendar_lookup_form",
        }
        headers = {"Content-Type": "application/x-www-form-urlencoded"}

        # This endpoint redirects to the data url.
        response = requests.request("POST", url, headers=headers, data=payload)
        bin_data_url = f"{response.url}/ajax"

        # Get bin data
        response = requests.request("GET", bin_data_url)
        bin_data = response.json()

        # Parse bin data
        soup = BeautifulSoup(bin_data[1]["data"], "html.parser")

        # All collection info is in the table
        table = (
            soup.find("div", {"id": "upcoming-collection"}).find("table").find("tbody")
        )
        rows = table.find_all("tr")

        data = {"bins": []}
        for row in rows:
            cols = row.find_all("td")
            # First column is date
            bin_date = datetime.strptime(cols[0].text.strip(), "%d %B %Y")
            # Third column is type
            bin_type = cols[2].text.strip()
            # This bin
            this_bin = {
                "type": bin_type,
                "collectionDate": bin_date.strftime(date_format),
            }
            data["bins"].append(this_bin)

        return data



import json
import math
from datetime import timedelta

import requests
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # I need to point out that this one gave me a good head scratch. Mainly because I wrote lots
        # of code to parse the form and all that, then realised this url returns json data... oops.
        base_url = "https://www.doncaster.gov.uk/Compass/PremiseDetail/GetCollectionsForCalendar"

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        # Working with epoch times, otherwise known as posix/unix timestamps. The number of weeks
        # to return can actually be customised in the below timedelta
        today = math.floor(datetime.today().timestamp())
        four_weeks = math.floor((datetime.today() + timedelta(days=4 * 7)).timestamp())

        # For some reason, the actual web form uses a property id that's completely different
        # from the uprn - luckily this one is easy to find!
        params = {
            "UPRN": user_uprn,
            "Start": str(today),
            "End": str(four_weeks),
        }

        requests.packages.urllib3.disable_warnings()
        response = requests.get(base_url, params=params)

        # 200 = ok. I got a 500 in testing, so assumed no data for that address
        if response.status_code != 200:
            raise ValueError("No bins found for provided UPRN.")

        # Load the json results
        json_results = json.loads(response.text)["slots"]

        data = {"bins": []}
        collections = []

        # Each item is a dictionary, so accessing is easy
        for item in json_results:
            bin_type = item["title"]

            # item["start"] actually returns a string, so we want to only take digits or +s.
            # OK, we don't actually want the +s... or anything on the end of them, that's why
            # we split the string then cast the remaining epoch to a float
            epoch = "".join([i for i in item["start"] if i.isdigit() or i == "+"])
            epoch = epoch.split("+")[0]
            epoch = float(epoch)
            bin_date = datetime.strptime(
                str(datetime.fromtimestamp(epoch / 1000)), "%Y-%m-%d %H:%M:%S"
            )
            collections.append((bin_type, bin_date))

            # This orders the data we just parsed to date order
            ordered_data = sorted(collections, key=lambda x: x[1])
            data = {"bins": []}
            for bin in ordered_data:
                dict_data = {
                    "type": bin[0],
                    "collectionDate": bin[1].strftime(date_format),
                }
                data["bins"].append(dict_data)

        return data



from datetime import datetime
from urllib.parse import urlparse

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        root_url = "https://my.rbwm.gov.uk"
        href_url = ""
        api_url = "https://my.rbwm.gov.uk/block_refresh/block/47/node/136968?"
        user_postcode = kwargs.get("postcode")
        user_paon = kwargs.get("paon")

        data = {"bins": []}

        requests.packages.urllib3.disable_warnings()
        s = requests.Session()
        # Form start
        headers = {
            "authority": "my.rbwm.gov.uk",
            "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8",
            "accept-language": "en-GB,en;q=0.8",
            "cache-control": "max-age=0",
            "referer": "https://my.rbwm.gov.uk/special/your-collection-dates?uprn=100080371082&subdate=2022-08-19&addr=11%20Douglas%20Lane%20Wraysbury%20Staines%20TW19%205NF",
            "sec-fetch-dest": "document",
            "sec-fetch-mode": "navigate",
            "sec-fetch-site": "same-origin",
            "sec-fetch-user": "?1",
            "sec-gpc": "1",
            "upgrade-insecure-requests": "1",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.5112.102 Safari/537.36",
        }
        s.get(
            "https://my.rbwm.gov.uk/special/find-your-collection-dates", headers=headers
        )

        # Select address
        headers = {
            "authority": "my.rbwm.gov.uk",
            "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8",
            "accept-language": "en-GB,en;q=0.8",
            "cache-control": "max-age=0",
            "origin": "https://my.rbwm.gov.uk",
            "referer": "https://my.rbwm.gov.uk/special/find-your-collection-dates",
            "sec-fetch-dest": "document",
            "sec-fetch-mode": "navigate",
            "sec-fetch-site": "same-origin",
            "sec-fetch-user": "?1",
            "sec-gpc": "1",
            "upgrade-insecure-requests": "1",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.5112.102 Safari/537.36",
        }
        request_data = {
            "atTxtStreet": user_postcode,
            "nodeid": "x",
            "formname": "x",
            "pg": "20",
            "start": "1",
            "selectaddress": "Select this address",
            "selectheading": "The following addresses match the address you entered - choose your address",
            "arg": "",
        }
        response = s.post(
            "https://my.rbwm.gov.uk/special/address-selector-collection-dates",
            headers=headers,
            data=request_data,
        )

        soup = BeautifulSoup(response.content, features="html.parser")
        soup.prettify()

        table = soup.find("table")
        if table:
            table_rows = table.find_all("tr")
            for tr in table_rows:
                td = tr.find_all("td")
                # row = [i.text for i in td]
                for item in td:
                    if user_paon in item.text and user_postcode in item.text:
                        href_url = td[1].find("a").get("href")
                        continue

            # Getting to bin data
            headers = {
                "authority": "my.rbwm.gov.uk",
                "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8",
                "accept-language": "en-GB,en;q=0.8",
                "referer": "https://my.rbwm.gov.uk/special/address-selector-collection-dates",
                "sec-fetch-dest": "document",
                "sec-fetch-mode": "navigate",
                "sec-fetch-site": "same-origin",
                "sec-fetch-user": "?1",
                "sec-gpc": "1",
                "upgrade-insecure-requests": "1",
                "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.5112.102 Safari/537.36",
            }
            params = {}
            parsed_params = urlparse(href_url).query.split("&")
            for item in parsed_params:
                values = item.split("=")
                params.update({values[0]: values[1]})

            s.get(root_url + href_url, params=params, headers=headers)
            response = s.get(
                api_url + href_url.split("?")[1], params=params, headers=headers
            )

            soup = BeautifulSoup(response.content, features="html.parser")
            soup.prettify()

            table_rows = soup.find_all("tr")
            for tr in table_rows:
                td = tr.find_all("td")
                row = [i.text for i in td]

                if len(row) > 0:
                    dict_data = {
                        "type": row[0],
                        "collectionDate": row[1],
                    }
                    data["bins"].append(dict_data)

        return data



from datetime import datetime

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = (
                "https://myselfservice.ne-derbyshire.gov.uk/service/Check_your_Bin_Day"
            )

            data = {"bins": []}

            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_uprn(user_uprn)
            check_postcode(user_postcode)
            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            # If you bang in the house number (or property name) and postcode in the box it should find your property

            iframe_presense = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located((By.ID, "fillform-frame-1"))
            )

            driver.switch_to.frame(iframe_presense)
            wait = WebDriverWait(driver, 60)
            inputElement_postcodesearch = wait.until(
                EC.element_to_be_clickable((By.NAME, "postcode_search"))
            )

            inputElement_postcodesearch.send_keys(str(user_postcode))

            # Wait for the 'Select your property' dropdown to appear and select the first result
            dropdown = wait.until(EC.element_to_be_clickable((By.NAME, "selAddress")))

            dropdown_options = wait.until(
                EC.presence_of_element_located((By.CLASS_NAME, "lookup-option"))
            )

            # Create a 'Select' for it, then select the first address in the list
            # (Index 0 is "Make a selection from the list")
            drop_down_values = Select(dropdown)
            option_element = wait.until(
                EC.presence_of_element_located(
                    (By.CSS_SELECTOR, f'option.lookup-option[value="{str(user_uprn)}"]')
                )
            )

            drop_down_values.select_by_value(str(user_uprn))

            # Wait for the 'View more' link to appear, then click it to get the full set of dates
            h3_element = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, "//th[contains(text(), 'Waste Collection')]")
                )
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            target_h3 = soup.find("h3", string="Collection Details")
            tables_after_h3 = target_h3.parent.parent.find_next("table")

            table_rows = tables_after_h3.find_all("tr")
            for row in table_rows:
                rowdata = row.find_all("td")
                if len(rowdata) == 3:
                    labels = rowdata[0].find_all("label")
                    # Strip the day (i.e., Monday) out of the collection date string for parsing
                    if len(labels) >= 2:
                        date_label = labels[1]
                        datestring = date_label.text.strip()

                    # Add the bin type and collection date to the 'data' dictionary
                    data["bins"].append(
                        {
                            "type": rowdata[2].text.strip(),
                            "collectionDate": datetime.strptime(
                                datestring, "%d/%m/%Y"
                            ).strftime(
                                date_format
                            ),  # Format the date as needed
                        }
                    )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import time
from datetime import datetime

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = "https://tendring-self.achieveservice.com/en/service/Rubbish_and_recycling_collection_days"

            bin_data = {"bins": []}

            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_uprn(user_uprn)
            check_postcode(user_postcode)
            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            cookies_button = WebDriverWait(driver, timeout=15).until(
                EC.presence_of_element_located((By.ID, "close-cookie-message"))
            )
            cookies_button.click()

            without_login_button = WebDriverWait(driver, timeout=15).until(
                EC.presence_of_element_located(
                    (By.LINK_TEXT, "or, continue without an account")
                )
            )
            without_login_button.click()

            iframe_presense = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located((By.ID, "fillform-frame-1"))
            )

            driver.switch_to.frame(iframe_presense)
            wait = WebDriverWait(driver, 60)
            inputElement_postcodesearch = wait.until(
                EC.element_to_be_clickable((By.NAME, "postcode_search"))
            )

            inputElement_postcodesearch.send_keys(user_postcode)

            # Wait for the 'Select address' dropdown to be updated
            time.sleep(1)

            dropdown = wait.until(
                EC.element_to_be_clickable((By.NAME, "selectAddress"))
            )
            # Create a 'Select' for it, then select the first address in the list
            # (Index 0 is "Select...")
            dropdownSelect = Select(dropdown)
            dropdownSelect.select_by_value(str(user_uprn))

            # Wait for 'wasteTable' to be shown
            wait.until(EC.presence_of_element_located((By.CLASS_NAME, "wasteTable")))

            soup = BeautifulSoup(driver.page_source, features="html.parser")
            bins = (
                soup.find("table", {"class": "wasteTable"}).find("tbody").find_all("tr")
            )
            for bin_row in bins:
                bin = bin_row.find_all("td")
                if bin:
                    if bin[1].get_text(strip=True) != "":
                        bin_date = datetime.strptime(
                            bin[1].get_text(strip=True), "%d/%m/%Y"
                        )
                        dict_data = {
                            "type": re.sub(
                                r"\([^)]*\)", "", bin[0].get_text(strip=True)
                            ),
                            "collectionDate": bin_date.strftime(date_format),
                        }
                        bin_data["bins"].append(dict_data)

            bin_data["bins"].sort(
                key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
            )

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return bin_data



from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the base
    class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            # Create Selenium webdriver
            headless = kwargs.get("headless")
            driver = create_webdriver(
                kwargs.get("web_driver"), headless, None, __name__
            )
            driver.get(kwargs.get("url"))

            # Make a BS4 object
            soup = BeautifulSoup(driver.page_source, features="html.parser")
            soup.prettify()

            data = {"bins": []}
            sections = soup.find_all("div", {"class": "wil_c-content-section_heading"})
            for s in sections:
                if s.get_text(strip=True).lower() == "bin collections":
                    rows = s.find_next_sibling(
                        "div", {"class": "c-content-section_body"}
                    ).find_all("div", {"class": "u-mb-8"})
                    for row in rows:
                        title = row.find("div", {"class": "u-mb-4"})
                        collections = row.find_all("div", {"class": "u-mb-2"})
                        if title and collections:
                            for c in collections:
                                if (
                                    c.get_text(strip=True)
                                    .lower()
                                    .startswith("next collection")
                                ):
                                    # add next collection
                                    next_collection_date = datetime.strptime(
                                        c.get_text(strip=True).replace(
                                            "Next collection - ", ""
                                        ),
                                        "%A, %d %B %Y",
                                    ).strftime(date_format)
                                    dict_data = {
                                        "type": title.get_text(strip=True).capitalize(),
                                        "collectionDate": next_collection_date,
                                    }
                                    data["bins"].append(dict_data)
                                    # add future collections without duplicating next collection
                                    future_collections = row.find(
                                        "ul", {"class": "u-mt-4"}
                                    ).find_all("li")
                                    for c in future_collections:
                                        future_collection_date = datetime.strptime(
                                            c.get_text(strip=True),
                                            "%A, %d %B %Y",
                                        ).strftime(date_format)
                                        if (
                                            future_collection_date
                                            != next_collection_date
                                        ):
                                            dict_data = {
                                                "type": title.get_text(
                                                    strip=True
                                                ).capitalize(),
                                                "collectionDate": future_collection_date,
                                            }
                                            data["bins"].append(dict_data)

            data["bins"].sort(
                key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
            )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        uprn = kwargs.get("uprn")
        check_uprn(uprn)

        # Make SOAP Request
        headers = {
            "Content-Type": "text/xml; charset=UTF-8",
            "SOAPAction": '"http://tempuri.org/GetCollections"',
        }

        post_data = (
            '<?xml version="1.0" encoding="utf-8"?><soap:Envelope '
            'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            'xmlns:xsd="http://www.w3.org/2001/XMLSchema" '
            'xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"><soap:Body><GetCollections '
            'xmlns="http://tempuri.org/"><uprn>'
            + uprn
            + "</uprn></GetCollections></soap:Body></soap:Envelope>"
        )

        response = requests.post(
            "http://avdcbins.web-labs.co.uk/RefuseApi.asmx",
            data=post_data,
            headers=headers,
        )

        if response.status_code != 200:
            raise ValueError("No collection data found for provided UPRN.")

        # Make a BS4 object
        soup = BeautifulSoup(response.text, "xml")
        soup.prettify()

        data = {"bins": []}

        all_collections = soup.find_all("BinCollection")

        for i in range(len(all_collections)):
            collection_date = datetime.strptime(
                all_collections[i].Date.get_text(), "%Y-%m-%dT%H:%M:%S"
            )
            children = all_collections[i].find_all(
                ["Refuse", "Recycling", "Garden", "Food"], string="true"
            )

            for collection in children:
                dict_data = {
                    "type": collection.name,
                    "collectionDate": collection_date.strftime(date_format),
                }
                data["bins"].append(dict_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )
        return data



import requests

from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        data = {"bins": []}
        api_url = f'https://portal.mansfield.gov.uk/MDCWhiteSpaceWebService/WhiteSpaceWS.asmx/GetCollectionByUPRNAndDate?apiKey=mDc-wN3-B0f-f4P&UPRN={user_uprn}&coldate={datetime.now().strftime("%d/%m/%Y")}'

        response = requests.get(api_url)
        if response.status_code != 200:
            raise ConnectionError("Could not get latest data!")

        json_data = response.json()["Collections"]
        for item in json_data:

            dict_data = {
                "type": item.get("Service").split(" ")[0] + " bin",
                "collectionDate": datetime.strptime(
                    item.get("Date"), "%d/%m/%Y %H:%M:%S"
                ).strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        uprn = kwargs.get("uprn")
        check_uprn(uprn)

        requests.packages.urllib3.disable_warnings()
        response = requests.get(
            f"https://map.erewash.gov.uk/isharelive.web/myerewash.aspx?action=SetAddress&UniqueId={uprn}",
            headers={"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64)"},
        )

        soup = BeautifulSoup(response.text, features="html.parser")
        collections = soup.find("div", {"aria-label": "Waste Collection"}).find_all(
            "div", {"class": "atPanelContent"}
        )
        for c in collections:
            bin_type = c.find("h4").get_text(strip=True)
            if "my next" in bin_type.lower():
                collection_info = c.find("div", {"class": "atPanelData"}).get_text(
                    strip=True
                )
                results = re.search(
                    "([A-Za-z]+ \\d+[A-Za-z]+ [A-Za-z]+ \\d*)", collection_info
                )
                if results:
                    collection_date = datetime.strptime(
                        remove_ordinal_indicator_from_date_string(results[1]).strip(),
                        "%A %d %B %Y",
                    ).strftime(date_format)
                    dict_data = {
                        "type": bin_type.replace("My Next ", "").replace(
                            " Collection", ""
                        ),
                        "collectionDate": collection_date,
                    }
                    data["bins"].append(dict_data)
                    if "garden waste" in collection_info.lower():
                        dict_data = {
                            "type": "Garden Waste",
                            "collectionDate": collection_date,
                        }
                        data["bins"].append(dict_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return data



from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def get_data(self, page) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        for month in soup.select('div[class*="bin-collection__month"]'):
            monthName = month.select('h3[class*="bin-collection__title"]')[
                0
            ].text.strip()
            for collectionDay in month.select('li[class*="bin-collection__item"]'):
                bin_type = collectionDay.select('span[class*="bin-collection__type"]')[
                    0
                ].text.strip()
                binCollection = (
                    collectionDay.select('span[class*="bin-collection__day"]')[
                        0
                    ].text.strip()
                    + ", "
                    + collectionDay.select('span[class*="bin-collection__number"]')[
                        0
                    ].text.strip()
                    + " "
                    + monthName
                )

                dict_data = {
                    "type": bin_type,
                    "collectionDate": datetime.strptime(
                        binCollection, "%A, %d %B %Y"
                    ).strftime(date_format),
                }

                data["bins"].append(dict_data)

        return data

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = "https://www.highpeak.gov.uk/findyourbinday"

            # Assign user info
            user_postcode = kwargs.get("postcode")
            user_paon = kwargs.get("paon")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            # Hide Cookies
            inputElement_hc = driver.find_element(
                By.CLASS_NAME, "cookiemessage__link--close"
            )
            inputElement_hc.click()

            # Enter postcode in text box and wait
            inputElement_pc = driver.find_element(
                By.ID, "FINDBINDAYSHIGHPEAK_POSTCODESELECT_POSTCODE"
            )
            inputElement_pc.send_keys(user_postcode)
            inputElement_pc.send_keys(Keys.ENTER)

            WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.ID, "FINDBINDAYSHIGHPEAK_ADDRESSSELECT_ADDRESS")
                )
            )

            # Select address from dropdown and wait
            inputElement_ad = Select(
                driver.find_element(By.ID, "FINDBINDAYSHIGHPEAK_ADDRESSSELECT_ADDRESS")
            )

            inputElement_ad.select_by_visible_text(user_paon)

            WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.ID,
                        "FINDBINDAYSHIGHPEAK_ADDRESSSELECT_ADDRESSSELECTNEXTBTN_NEXT",
                    )
                )
            )

            # Submit address information and wait
            driver.find_element(
                By.ID, "FINDBINDAYSHIGHPEAK_ADDRESSSELECT_ADDRESSSELECTNEXTBTN_NEXT"
            ).click()

            WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.ID, "FINDBINDAYSHIGHPEAK_CALENDAR_MAINCALENDAR")
                )
            )

            # Read next collection information into Pandas
            table = driver.find_element(
                By.ID, "FINDBINDAYSHIGHPEAK_CALENDAR_MAINCALENDAR"
            ).get_attribute("outerHTML")

            # Parse data into dict
            data = self.get_data(table)
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = "https://forms.north-norfolk.gov.uk/outreach/BinCollectionDays.ofml"

            data = {"bins": []}

            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_paon(user_paon)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            # Populate postcode field
            inputElement_postcode = driver.find_element(
                By.ID,
                "F_Address_subform:Postcode",
            )
            inputElement_postcode.send_keys(user_postcode)

            # Click search button
            driver.find_element(
                By.ID,
                "BA_Address_subform:Search_button",
            ).click()

            # Wait for the 'Select address' dropdown to appear
            dropdown = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.XPATH, "//select[@id='F_Address_subform:Id']")
                )
            )
            # Create a 'Select' for it, then select the matching house number/name option
            dropdownSelect = Select(dropdown)
            matchingOptions = [
                o for o in dropdownSelect.options if user_paon.lower() in o.text.lower()
            ]
            if matchingOptions:
                matchingOptions[0].click()

                # Wait for the results to appear
                WebDriverWait(driver, 10).until(
                    EC.presence_of_element_located(
                        (By.XPATH, "//div[contains(@class, 'fieldmergedcolumn')]/ul")
                    )
                )

                soup = BeautifulSoup(driver.page_source, features="html.parser")

                bins_text = soup.find("div", id="Search_result_details_cps_hd")

                if bins_text:
                    results = re.findall(
                        "Your next (.*?) Bin collection is ([A-Za-z]+ \\d\\d? [A-Za-z]+)",
                        bins_text.get_text(),
                    )
                    if results:
                        for result in results:
                            collection_date = datetime.strptime(
                                result[1] + " " + datetime.now().strftime("%Y"),
                                "%A %d %B %Y",
                            )
                            dict_data = {
                                "type": result[0],
                                "collectionDate": collection_date.strftime(date_format),
                            }
                            data["bins"].append(dict_data)

                            data["bins"].sort(
                                key=lambda x: datetime.strptime(
                                    x.get("collectionDate"), date_format
                                )
                            )
            else:
                raise ValueError("No matching address for house number/name found.")
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import re  # Import regular expressions
from datetime import datetime, timedelta

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}

            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_uprn(user_uprn)
            check_postcode(user_postcode)
            # Create Selenium webdriver
            page = f"https://my.nwleics.gov.uk/my-property-finder?address={user_postcode}&go=1"

            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            # If you bang in the house number (or property name) and postcode in the box it should find your property

            # iframe_presense = WebDriverWait(driver, 30).until(
            #    EC.presence_of_element_located((By.ID, "fillform-frame-1"))
            # )

            # driver.switch_to.frame(iframe_presense)
            wait = WebDriverWait(driver, 60)

            address_link = wait.until(
                EC.element_to_be_clickable(
                    (By.XPATH, f'//a[contains(@href, "{user_uprn}")]')
                )
            )

            address_link.click()

            refuse_element = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, f'//h3[contains(text(), "Refuse Collection Dates")]')
                )
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            # Find the unordered list containing refuse collection details
            refuse_list = soup.find("ul", class_="refuse")

            current_year = datetime.now().year

            if refuse_list:
                # Iterate through list items within the unordered list
                for li in refuse_list.find_all("li"):
                    date = li.find(
                        "strong", class_="date"
                    ).text.strip()  # Extract the date
                    waste_type = li.find("a").text.strip()  # Extract the waste type

                    # Parse the date from the string
                    # check for today and tomorrow
                    if date.lower() == "today":
                        parsed_date = datetime.now().date()
                    elif date.lower() == "tomorrow":
                        parsed_date = (datetime.now() + timedelta(days=1)).date()
                    else:
                        date = re.sub(r"(st|nd|rd|th)", "", date)
                        parsed_date = datetime.strptime(date, "%a %d %b").date()

                    current_date = datetime.now().date()

                    # double check we've got a year and if not the current one
                    if parsed_date.year < current_date.year:
                        parsed_date = parsed_date.replace(year=current_date.year)

                    # check if the date is in the past and if so add a year
                    if parsed_date < current_date:
                        parsed_date = parsed_date.replace(year=current_date.year + 1)

                    # Append data to your 'bins' list (this replicates your existing logic)
                    data["bins"].append(
                        {
                            "type": waste_type,
                            "collectionDate": parsed_date.strftime("%d/%m/%Y"),
                        }
                    )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import time

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select, WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


def get_seasonal_overrides():
    url = "https://www.barnet.gov.uk/recycling-and-waste/bin-collections/find-your-bin-collection-day"
    response = requests.get(url)
    if response.status_code == 200:
        soup = BeautifulSoup(response.text, "html.parser")
        body_div = soup.find("div", class_="field--name-body")
        ul_element = body_div.find("ul")
        if ul_element:
            li_elements = ul_element.find_all("li")
            overrides_dict = {}
            for li_element in li_elements:
                li_text = li_element.text.strip()
                li_text = re.sub(r"\([^)]*\)", "", li_text).strip()
                if "Collections for" in li_text and "will be revised to" in li_text:
                    parts = li_text.split("will be revised to")
                    original_date = (
                        parts[0]
                        .replace("Collections for", "")
                        .replace("\xa0", " ")
                        .strip()
                    )
                    revised_date = parts[1].strip()

                    # Extract day and month
                    date_parts = original_date.split()[1:]
                    if len(date_parts) == 2:
                        day, month = date_parts
                        # Ensure original_date has leading zeros for single-digit days
                        day = day.zfill(2)
                        original_date = f"{original_date.split()[0]} {day} {month}"

                    # Store the information in the dictionary
                    overrides_dict[original_date] = revised_date
            return overrides_dict
        else:
            print("UL element not found within the specified div.")
    else:
        print(f"Failed to retrieve the page. Status code: {response.status_code}")


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            user_postcode = kwargs.get("postcode")
            if not user_postcode:
                raise ValueError("No postcode provided.")
            check_postcode(user_postcode)

            user_paon = kwargs.get("paon")
            check_paon(user_paon)
            headless = kwargs.get("headless")
            web_driver = kwargs.get("web_driver")
            driver = create_webdriver(web_driver, headless, None, __name__)
            page = "https://account.barnet.gov.uk/Forms/Home/Redirector/Index/?id=6a2ac067-3322-46e5-96e4-16c0c214454a&mod=OA&casetype=BAR&formname=BNTCOLDATE"
            driver.get(page)

            time.sleep(5)

            try:
                accept_cookies = WebDriverWait(driver, timeout=10).until(
                    EC.presence_of_element_located((By.ID, "epdagree"))
                )
                accept_cookies.click()
                accept_cookies_submit = WebDriverWait(driver, timeout=10).until(
                    EC.presence_of_element_located((By.ID, "epdsubmit"))
                )
                accept_cookies_submit.click()
            except:
                print(
                    "Accept cookies banner not found or clickable within the specified time."
                )
                pass

            postcode_input = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.CSS_SELECTOR, '[aria-label="Postcode"]')
                )
            )

            postcode_input.send_keys(user_postcode)

            find_address_button = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.CSS_SELECTOR, '[value="Find address"]')
                )
            )
            find_address_button.click()

            time.sleep(15)
            # Wait for address box to be visible
            select_address_input = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.ID,
                        "MainContent_CUSTOM_FIELD_808562d4b07f437ea751317cabd19d9eeaf8742f49cb4f7fa9bef99405b859f2",
                    )
                )
            )

            # Select address based
            select = Select(select_address_input)
            addr_label = f"{user_postcode}, {user_paon},"
            for addr_option in select.options:
                option_name = addr_option.accessible_name[0 : len(addr_label)]
                if option_name == addr_label:
                    break
            select.select_by_value(addr_option.text)

            time.sleep(10)
            # Wait for the specified div to be present
            target_div_id = "MainContent_CUSTOM_FIELD_808562d4b07f437ea751317cabd19d9ed93a174c32b14f839b65f6abc42d8108_div"
            target_div = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located((By.ID, target_div_id))
            )

            time.sleep(5)
            soup = BeautifulSoup(driver.page_source, "html.parser")

            # Find the div with the specified id
            target_div = soup.find("div", {"id": target_div_id})

            # Handle the additional table of info for xmas
            try:
                overrides_dict = get_seasonal_overrides()
            except Exception as e:
                overrides_dict = {}

            # Check if the div is found
            if target_div:
                bin_data = {"bins": []}

                for bin_div in target_div.find_all(
                    "div",
                    {"style": re.compile("background-color:.*; padding-left: 4px;")},
                ):
                    bin_type = bin_div.find("strong").text.strip()
                    collection_date_string = (
                        re.search(r"Next collection date:\s+(.*)", bin_div.text)
                        .group(1)
                        .strip()
                        .replace(",", "")
                    )
                    if collection_date_string in overrides_dict:
                        # Replace with the revised date from overrides_dict
                        collection_date_string = overrides_dict[collection_date_string]

                    current_date = datetime.now()
                    parsed_date = datetime.strptime(
                        collection_date_string + f" {current_date.year}", "%A %d %B %Y"
                    )
                    # Check if the parsed date is in the past and not today
                    if parsed_date.date() < current_date.date():
                        # If so, set the year to the next year
                        parsed_date = parsed_date.replace(year=current_date.year + 1)
                    else:
                        # If not, set the year to the current year
                        parsed_date = parsed_date.replace(year=current_date.year)
                    formatted_date = parsed_date.strftime("%d/%m/%Y")

                    contains_date(formatted_date)
                    bin_info = {"type": bin_type, "collectionDate": formatted_date}
                    bin_data["bins"].append(bin_info)
            else:
                raise ValueError("Collection data not found.")

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return bin_data



import argparse
import importlib
import os
import sys
import logging
from uk_bin_collection.uk_bin_collection.get_bin_data import (
    setup_logging,
    LOGGING_CONFIG,
)

_LOGGER = logging.getLogger(__name__)


def import_council_module(module_name, src_path="councils"):
    """Dynamically import the council processor module."""
    module_path = os.path.realpath(os.path.join(os.path.dirname(__file__), src_path))
    if module_path not in sys.path:
        sys.path.append(module_path)
    return importlib.import_module(module_name)


class UKBinCollectionApp:
    def __init__(self):
        self.setup_arg_parser()
        self.parsed_args = None

    def setup_arg_parser(self):
        """Setup the argument parser for the script."""
        self.parser = argparse.ArgumentParser(
            description="UK Bin Collection Data Parser"
        )
        self.parser.add_argument(
            "module", type=str, help="Name of council module to use"
        )
        self.parser.add_argument(
            "URL", type=str, help="URL to parse - should be wrapped in double quotes"
        )
        self.parser.add_argument(
            "-p",
            "--postcode",
            type=str,
            help="Postcode to parse - should include a space and be wrapped in double quotes",
            required=False,
        )
        self.parser.add_argument(
            "-n", "--number", type=str, help="House number to parse", required=False
        )
        self.parser.add_argument(
            "-s",
            "--skip_get_url",
            action="store_true",
            help="Skips the generic get_url - uses one in council class",
            required=False,
        )
        self.parser.add_argument(
            "-u", "--uprn", type=str, help="UPRN to parse", required=False
        )
        self.parser.add_argument(
            "-w",
            "--web_driver",
            type=str,
            help="URL for remote Selenium web driver - should be wrapped in double quotes",
            required=False,
        )
        self.parser.add_argument(
            "--headless",
            dest="headless",
            action="store_true",
            help="Should Selenium be headless. Defaults to true. Can be set to false to debug council",
        )
        self.parser.add_argument(
            "--not-headless",
            dest="headless",
            action="store_false",
            help="Should Selenium be headless. Defaults to true. Can be set to false to debug council",
        )
        self.parser.set_defaults(headless=True)
        self.parser.add_argument(
            "--local_browser",
            dest="local_browser",
            action="store_true",
            help="Should Selenium be run on a remote server or locally. Defaults to false.",
            required=False,
        )
        self.parser.add_argument(
            "-d",
            "--dev_mode",
            action="store_true",
            help="Enables development mode - creates/updates entries in the input.json file for the council on each run",
            required=False,
        )
        self.parsed_args = None

    def set_args(self, args):
        """Parse the arguments from the command line."""
        self.parsed_args = self.parser.parse_args(args)

    def run(self):
        """Run the application with the provided arguments."""
        council_module = import_council_module(self.parsed_args.module)
        return self.client_code(
            council_module.CouncilClass(),
            self.parsed_args.URL,
            postcode=self.parsed_args.postcode,
            paon=self.parsed_args.number,
            uprn=self.parsed_args.uprn,
            skip_get_url=self.parsed_args.skip_get_url,
            web_driver=self.parsed_args.web_driver,
            headless=self.parsed_args.headless,
            local_browser=self.parsed_args.local_browser,
            dev_mode=self.parsed_args.dev_mode,
            council_module_str=self.parsed_args.module,
        )

    def client_code(self, get_bin_data_class, address_url, **kwargs):
        """
        Call the template method to execute the algorithm. Client code does not need
        to know the concrete class of an object it works with, as long as it works with
        objects through the interface of their base class.
        """
        return get_bin_data_class.template_method(address_url, **kwargs)


def run():
    """Set up logging and run the application."""
    global _LOGGER
    _LOGGER = setup_logging(LOGGING_CONFIG, None)
    app = UKBinCollectionApp()
    app.set_args(sys.argv[1:])
    print(app.run())


if __name__ == "__main__":
    run()



from bs4 import BeautifulSoup, element
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        collections = []
        url_base = "https://geoapi.dorsetcouncil.gov.uk/v1/services/"
        url_types = ["recyclingday", "refuseday", "foodwasteday", "gardenwasteday"]

        uprn = kwargs.get("uprn")
        # Check the UPRN is valid
        check_uprn(uprn)

        for url_type in url_types:
            response = requests.get(f"{url_base}{url_type}/{uprn}")
            if response.status_code != 200:
                raise ConnectionError(f"Could not fetch from {url_type} endpoint")
            json_data = response.json()["values"][0]
            collections.append((f"{json_data.get('type')} bin", datetime.strptime(json_data.get('dateNextVisit'), "%Y-%m-%d")))

        # Sort the text and date elements by date
        ordered_data = sorted(collections, key=lambda x: x[1])

        # Put the elements into the dictionary
        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass
from dateutil.relativedelta import relativedelta



# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        collections = []

        curr_date = datetime.today()

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        headers = {
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
            'Accept-Language': 'en-GB,en;q=0.9',
            'Cache-Control': 'no-cache',
            'Connection': 'keep-alive',
            'Pragma': 'no-cache',
            'Sec-Fetch-Dest': 'document',
            'Sec-Fetch-Mode': 'navigate',
            'Sec-Fetch-Site': 'none',
            'Sec-Fetch-User': '?1',
            'Upgrade-Insecure-Requests': '1',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.6422.143 Safari/537.36',
            'sec-ch-ua': '"Opera GX";v="111", "Chromium";v="125", "Not.A/Brand";v="24"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"',
        }
        params = {
            'uprn': f'{user_uprn}',
            # 'uprn': f'100040128734',
        }
        response = requests.get(
            'https://www.cornwall.gov.uk/umbraco/surface/waste/MyCollectionDays',
            params=params,
            headers=headers
        )

        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        for item in soup.find_all('div', class_='collection text-center service'):
            bin_type = item.contents[1].text + " bin"
            collection_date = datetime.strptime(item.contents[5].text, "%d %b").replace(year=curr_date.year)
            if curr_date.month == 12 and collection_date.month == 1:
                collection_date = collection_date + relativedelta(years=1)
            collections.append((bin_type, collection_date))

            ordered_data = sorted(collections, key=lambda x: x[1])
            data = {"bins": []}
            for bin in ordered_data:
                dict_data = {
                    "type": bin[0].capitalize().strip(),
                    "collectionDate": bin[1].strftime(date_format),
                }
                data["bins"].append(dict_data)

        return data



from datetime import datetime

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Get and check UPRN
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        # Start a new session to walk through the form
        requests.packages.urllib3.disable_warnings()
        s = requests.Session()

        # There's a cookie that makes the whole thing valid when you search for a postcode,
        # but postcode and UPRN is a hassle imo, so this makes a request for the session to get a cookie
        # using a Manchester City Council postcode I hardcoded in the data payload
        postcode_request_header = {
            "authority": "www.manchester.gov.uk",
            "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,"
            "image/webp,image/apng,*/*;q=0.8",
            "accept-language": "en-GB,en;q=0.6",
            "cache-control": "max-age=0",
            # Requests sorts cookies= alphabetically
            "origin": "https://www.manchester.gov.uk",
            "referer": "https://www.manchester.gov.uk/bincollections",
            "sec-fetch-dest": "document",
            "sec-fetch-mode": "navigate",
            "sec-fetch-site": "same-origin",
            "sec-fetch-user": "?1",
            "sec-gpc": "1",
            "upgrade-insecure-requests": "1",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, "
            "like Gecko) Chrome/104.0.5112.102 Safari/537.36",
        }
        postcode_request_data = {
            "mcc_bin_dates_search_term": "M2 5DB",
            "mcc_bin_dates_submit": "Go",
        }
        response = s.post(
            "https://www.manchester.gov.uk/bincollections",
            headers=postcode_request_header,
            data=postcode_request_data,
        )

        # Make a POST with the same cookie-fied session using the user's UPRN data
        uprn_request_headers = {
            "authority": "www.manchester.gov.uk",
            "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8",
            "accept-language": "en-GB,en;q=0.6",
            "cache-control": "max-age=0",
            # Requests sorts cookies= alphabetically
            # 'cookie': 'TestCookie=Test; CookieConsent={stamp:%27D8rypjMDBJhpfMWybSMdGXP1hCZWGJYtGETiMTu1UuXTdRIKl8SU5g==%27%2Cnecessary:true%2Cpreferences:true%2Cstatistics:true%2Cmarketing:true%2Cver:6%2Cutc:1661783732090%2Cregion:%27gb%27}; PHPSESSID=kElJxYAt%2Cf-4ZWoskt0s5tn32BUQRXDYUVp3G-NsqOAOaeIcKlm2T4r7ATSgqfz6',
            "origin": "https://www.manchester.gov.uk",
            "referer": "https://www.manchester.gov.uk/bincollections",
            "sec-fetch-dest": "document",
            "sec-fetch-mode": "navigate",
            "sec-fetch-site": "same-origin",
            "sec-fetch-user": "?1",
            "sec-gpc": "1",
            "upgrade-insecure-requests": "1",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.5112.102 Safari/537.36",
        }
        uprn_request_data = {
            "mcc_bin_dates_uprn": user_uprn,
            "mcc_bin_dates_submit": "Go",
        }
        response = s.post(
            "https://www.manchester.gov.uk/bincollections",
            headers=uprn_request_headers,
            data=uprn_request_data,
        )

        # Make that BS4 object and use it to prettify the response
        soup = BeautifulSoup(response.content, features="html.parser")
        soup.prettify()

        # Get the collection items on the page and strip the bits of text that we don't care for
        collections = []
        for bin in soup.find_all("div", {"class": "collection"}):
            bin_type = bin.find_next("h3").text.replace("  DUE TODAY", "").strip()
            next_collection = bin.find_next("p").text.replace("Next collection ", "")
            next_collection = datetime.strptime(next_collection, "%A %d %b %Y")
            collections.append((bin_type, next_collection))

        # Sort the collections by date order rather than bin type, then return as a dictionary (with str date)
        ordered_data = sorted(collections, key=lambda x: x[1])
        data = {"bins": []}
        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



import logging
import pickle
import time

import requests
from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from uk_bin_collection.uk_bin_collection.common import *

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# Set up logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)


class CouncilClass(AbstractGetBinDataClass):

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_postcode(user_postcode)
            url = "https://bins.uttlesford.gov.uk/"

            # Get session cookies using requests

            user_agent = """general.useragent.override", "userAgent=Mozilla/5.0 
            (iPhone; CPU iPhone OS 15_4 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like 
            Gecko) CriOS/101.0.4951.44 Mobile/15E148 Safari/604.1"""

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, user_agent, __name__)

            # Navigate to the page first
            driver.get(url)

            wait = WebDriverWait(driver, 60)

            logging.info("Entering postcode")
            input_element_postcode = wait.until(
                EC.presence_of_element_located((By.XPATH, '//input[@id="postcode"]'))
            )

            input_element_postcode.send_keys(user_postcode)
            input_element_postcode.send_keys(Keys.ENTER)

            logging.info("Searching for postcode")
            input_element_postcode_dd = wait.until(
                EC.element_to_be_clickable((By.XPATH, '//select[@id="housenn"]'))
            )

            logging.info("Selecting address")
            drop_down_values = Select(input_element_postcode_dd)

            drop_down_values.select_by_visible_text(str(user_paon))

            input_element_address_btn = wait.until(
                EC.element_to_be_clickable(
                    (By.XPATH, '//input[@alt="View your waste collection days"]')
                )
            )

            input_element_address_btn.click()

            logging.info("Waiting for bin collection page")
            h3_element = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, "//h3[contains(text(), 'Future Collection Dates')]")
                )
            )

            logging.info("Extracting bin collection data")
            soup = BeautifulSoup(driver.page_source, features="html.parser")

            bins = []
            rows = soup.select("div.wrap table tbody tr")

            for row in rows:
                cols = row.find_all("td")
                if len(cols) == 2:
                    bin_types = [img["alt"] for img in cols[0].find_all("img")]
                    collection_date_str = cols[1].text
                    collection_date = datetime.strptime(
                        collection_date_str, "%A %dth %B"
                    )
                    collection_date = collection_date.replace(
                        year=2024
                    )  # Assuming the year is 2024
                    collection_date_str = collection_date.strftime("%d/%m/%Y")

                    for bin_type in bin_types:
                        bins.append(
                            {"type": bin_type, "collectionDate": collection_date_str}
                        )

            bin_data = {"bins": bins}

            return bin_data

        except Exception as e:
            logging.error(f"An error occurred: {e}")
            raise

        finally:
            if driver:
                driver.quit()
